'use strict';

Object.defineProperty(exports, '__esModule', { value: true });

var common = require('@lezer/common');
var lezerJulia = require('@plutojl/lezer-julia');
var language$1 = require('@codemirror/language');
var highlight = require('@lezer/highlight');
var autocomplete = require('@codemirror/autocomplete');

function _interopNamespace(e) {
    if (e && e.__esModule) return e;
    var n = Object.create(null);
    if (e) {
        Object.keys(e).forEach(function (k) {
            if (k !== 'default') {
                var d = Object.getOwnPropertyDescriptor(e, k);
                Object.defineProperty(n, k, d.get ? d : {
                    enumerable: true,
                    get: function () { return e[k]; }
                });
            }
        });
    }
    n["default"] = e;
    return Object.freeze(n);
}

var autocomplete__namespace = /*#__PURE__*/_interopNamespace(autocomplete);

function delimitedStrategy(context, units, closing) {
    let after = context.textAfter;
    let space = after.match(/^\s*/)[0].length;
    let closed = false;
    switch (closing.length) {
        case 1:
            closed = after.slice(space, space + closing[0].length) === closing[0];
            break;
        case 2:
            closed =
                after.slice(space, space + closing[0].length) === closing[0] ||
                    after.slice(space, space + closing[1].length) === closing[1];
            break;
        case 3:
            closed =
                after.slice(space, space + closing[0].length) === closing[0] ||
                    after.slice(space, space + closing[1].length) === closing[1] ||
                    after.slice(space, space + closing[2].length) === closing[2];
            break;
        default:
            closed = closing.some((closing) => after.slice(space, space + closing.length) === closing);
            break;
    }
    return context.baseIndent + (closed ? 0 : context.unit * units);
}
function delimitedIndent({ closing, units = 1, }) {
    return (context) => delimitedStrategy(context, units, closing);
}
function noIndent(context) {
    return context.baseIndent;
}

function getSyntaxConfig() {
    let syntaxConfig = {
        indents: {
            VariableDeclaration: language$1.continuedIndent(),
            AssignmentExpression: language$1.continuedIndent(),
        },
        keywords: [],
    };
    for (let node of lezerJulia.parser.nodeSet.types) {
        // Collect keywords
        let groups = node.prop(common.NodeProp.group);
        let group = groups != null ? groups[0] : null;
        if (group === "keyword") {
            syntaxConfig.keywords.push(node);
        }
        // Configure indents
        let nodeIndent;
        let closedBy = node.prop(common.NodeProp.closedBy);
        if (closedBy) {
            nodeIndent = delimitedIndent({ closing: closedBy });
        }
        else {
            nodeIndent = noIndent;
        }
        syntaxConfig.indents[node.name] = nodeIndent;
    }
    return syntaxConfig;
}
let syntaxConfig = getSyntaxConfig();
console.log(`syntaxConfig.keywords.map((t) => t.name):`, syntaxConfig.keywords.map((t) => t.name));
let juliaStyleTags = highlight.styleTags({
    String: highlight.tags.string,
    TripleString: highlight.tags.string,
    CommandString: highlight.tags.string,
    StringWithoutInterpolation: highlight.tags.string,
    TripleStringWithoutInterpolation: highlight.tags.string,
    CommandStringWithoutInterpolation: highlight.tags.string,
    "String/$ TripleString/$ CommandString/$": highlight.tags.special(highlight.tags.brace),
    "String/( TripleString/( CommandString/(": highlight.tags.special(highlight.tags.brace),
    "String/) TripleString/) CommandString/)": highlight.tags.special(highlight.tags.brace),
    Comment: highlight.tags.lineComment,
    BlockComment: highlight.tags.comment,
    "mutable struct StructDefinition/end": highlight.tags.definitionKeyword,
    "primitive type PrimitiveDefinition/end": highlight.tags.definitionKeyword,
    "const local global": highlight.tags.definitionKeyword,
    // "module ModuleDefinition/end import using export": t.moduleKeyword,
    "ForStatement/for ForBinding/in ForStatement/end": highlight.tags.controlKeyword,
    "WhileStatement/while WhileStatement/end": highlight.tags.controlKeyword,
    "IfClause/if IfClause/elseif ElseClause/else IfStatement/end": highlight.tags.controlKeyword,
    "default break return": highlight.tags.controlKeyword,
    "TryStatement/try CatchClause/catch FinallyClause/finally TryStatement/end": highlight.tags.controlKeyword,
    "( )": highlight.tags.paren,
    "[ ]": highlight.tags.paren,
    "{ }": highlight.tags.paren,
    [syntaxConfig.keywords.map((t) => t.name).join(" ")]: highlight.tags.keyword,
    BooleanLiteral: highlight.tags.bool,
    Number: highlight.tags.number,
    "Coefficient/PrefixedString!": highlight.tags.unit,
    // Look at us being rascals
    "Type! TypeParameters!": highlight.tags.typeName,
    // "StructDefinition/Definition! PrimitiveDefinition/Definition! AbstractDefinition/Definition!":
    //   t.definition(t.typeName),
    "StructDefinition/Identifier StructDefinition/AssignmentExpression/Identifier StructDefinition/TypedExpression/Identifier StructDefinition/AssignmentExpression/TypedExpression/Identifier": highlight.tags.definition(highlight.tags.propertyName),
    ":: <:": highlight.tags.typeOperator,
    Identifier: highlight.tags.variableName,
    "MacroIdentifier! MacroFieldExpression!": highlight.tags.macroName,
    "MacroDefinition/Definition!": highlight.tags.definition(highlight.tags.macroName),
    "FieldName!": highlight.tags.propertyName,
    FieldExpression: highlight.tags.propertyName,
    "FieldExpression .": highlight.tags.derefOperator,
    "Symbol!": highlight.tags.atom,
});
let language = language$1.LRLanguage.define({
    parser: lezerJulia.parser.configure({
        props: [
            juliaStyleTags,
            language$1.indentNodeProp.add(Object.assign(Object.assign({}, syntaxConfig.indents), { ModuleDefinition: noIndent, BareModuleDefinition: noIndent, VariableDeclaration: language$1.continuedIndent(), AssignmentExpression: language$1.continuedIndent() })),
        ],
    }),
    languageData: {
        commentTokens: { line: "#" },
        indentOnInput: /^\s*(\]|\}|\)|end|else|elseif|catch|finally)/,
        closeBrackets: { brackets: ["(", "[", "{", "'", '"', "`"] },
    },
});
const keywordCompletion = language.data.of({
    autocomplete: autocomplete__namespace.completeFromList(syntaxConfig.keywords.map((keyword) => ({
        label: keyword.name,
        type: "keyword",
    }))),
});
let defaultConfig = {
    enableKeywordCompletion: false,
};
function julia(config = defaultConfig) {
    config = Object.assign(Object.assign({}, defaultConfig), config);
    let extensions = [];
    if (config.enableKeywordCompletion) {
        extensions.push(keywordCompletion);
    }
    return new language$1.LanguageSupport(language, extensions);
}

exports.julia = julia;
exports.keywordCompletion = keywordCompletion;
